@@ -56,6 +56,7 @@ static gpointer default_home_location_callback           (void);
 static gpointer default_default_folder_viewer_callback	 (void);
 static void     import_old_preferences_if_needed         (void);
 static gpointer default_home_link_name                   (void);
+static gpointer default_computer_link_name               (void);
 static gpointer default_trash_link_name                  (void);
 
 /* An enumeration used for installing type specific preferences defaults. */
@@ -393,10 +394,6 @@ static const PreferenceDefault preference_defaults[] = {
 	  NULL, NULL,
 	  "search_bar_type"
 	},
-	{ NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW,
-	  PREFERENCE_BOOLEAN,
-	  GINT_TO_POINTER (FALSE)
-	},
 	{ NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS,
 	  PREFERENCE_STRING_LIST,
 	  "size,date_modified,type",
@@ -412,10 +409,6 @@ static const PreferenceDefault preference_defaults[] = {
 	  PREFERENCE_INTEGER,
 	  GINT_TO_POINTER (148)
 	},
-	{ NAUTILUS_PREFERENCES_START_WITH_TOOLBAR,
-	  PREFERENCE_BOOLEAN,
-	  GINT_TO_POINTER (TRUE)
-	},
 	{ NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR,
 	  PREFERENCE_BOOLEAN,
 	  GINT_TO_POINTER (TRUE)
@@ -496,7 +489,7 @@ static const PreferenceDefault preference_defaults[] = {
 	/* Desktop Preferences */
 	{ NAUTILUS_PREFERENCES_DESKTOP_HOME_VISIBLE,
 	  PREFERENCE_BOOLEAN,
-	  GINT_TO_POINTER (TRUE)
+	  GINT_TO_POINTER (FALSE)
 	},
 	
 	{ NAUTILUS_PREFERENCES_DESKTOP_HOME_NAME,
@@ -505,6 +498,17 @@ static const PreferenceDefault preference_defaults[] = {
 	  default_home_link_name, g_free,
 	},
 	
+	{ NAUTILUS_PREFERENCES_DESKTOP_COMPUTER_VISIBLE,
+	  PREFERENCE_BOOLEAN,
+	  GINT_TO_POINTER (TRUE)
+	},
+	
+	{ NAUTILUS_PREFERENCES_DESKTOP_COMPUTER_NAME,
+	  PREFERENCE_STRING,
+	  NULL,
+	  default_computer_link_name, g_free,
+	},
+	
 	{ NAUTILUS_PREFERENCES_DESKTOP_TRASH_VISIBLE,
 	  PREFERENCE_BOOLEAN,
 	  GINT_TO_POINTER (TRUE)
@@ -539,6 +543,12 @@ default_home_link_name (void)
 	return g_strdup_printf (_("%s's Home"), g_get_user_name ());
 }
 
+static gpointer
+default_computer_link_name (void)
+{
+	return g_strdup (_("Computer"));
+}
+
 static gpointer
 default_trash_link_name (void)
 {
@@ -809,14 +819,16 @@ import_old_preferences_if_needed (void)
 	for (i=0; OLD_PREFERENCES_TO_IMPORT[i].new_key != NULL; i++) {
 		to_import = & (OLD_PREFERENCES_TO_IMPORT[i]);
 
-		error = NULL;
-		value = gconf_client_get (client, to_import->old_key, &error);
+		if (gconf_client_key_is_writable (client, to_import->new_key, NULL)) {
+			error = NULL;
+			value = gconf_client_get (client, to_import->old_key, &error);
 
-		if ((error == NULL) && (value != NULL)) {
-			gconf_client_set (client, to_import->new_key, value, NULL);
-		} else {
-			if (error != NULL) {
-				g_error_free (error);
+			if ((error == NULL) && (value != NULL)) {
+				gconf_client_set (client, to_import->new_key, value, NULL);
+			} else {
+				if (error != NULL) {
+					g_error_free (error);
+				}
 			}
 		}
 	}
