@@ -630,7 +630,7 @@ real_adding_file (FMListView *view, NautilusFile *file)
 	 * and won't be specific to the search directory. Is that OK? 
 	 */
 
-	g_signal_connect_object (GTK_OBJECT (file), "changed",
+	g_signal_connect_object (G_OBJECT (file), "changed",
 				 G_CALLBACK (fm_directory_view_queue_file_change),
 				 view, G_CONNECT_SWAPPED);
 
@@ -800,9 +800,12 @@ reveal_selected_items_callback (BonoboUIComponent *component, gpointer user_data
 			parent_uri = nautilus_file_get_parent_uri (file);
 			if (parent_uri != NULL) {
 				file_as_list = g_list_prepend (NULL, nautilus_file_get_uri (file));
-				nautilus_view_open_location_force_new_window
+				/* FIXME: match the current window type */
+				nautilus_view_open_location
 					(fm_directory_view_get_nautilus_view (directory_view), 
 					 parent_uri,
+					 Nautilus_ViewFrame_OPEN_IN_SPATIAL,
+					 0,
 					 file_as_list);
 				eel_g_list_free_deep (file_as_list);
 			}
