@@ -65,6 +65,7 @@
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
@@ -72,7 +73,6 @@
 #include <libnautilus-private/nautilus-sound.h>
 #include <libnautilus-private/nautilus-bonobo-extensions.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
-#include <libnautilus-private/nautilus-volume-monitor.h>
 #include <libnautilus-private/nautilus-desktop-link-monitor.h>
 #include <libnautilus-private/nautilus-directory-private.h>
 #include <bonobo-activation/bonobo-activation.h>
@@ -97,8 +97,11 @@ static GList *nautilus_application_spatial_window_list;
 static gboolean need_to_show_first_time_druid     (void);
 static void     desktop_changed_callback          (gpointer                  user_data);
 static void     desktop_location_changed_callback (gpointer                  user_data);
-static void     volume_unmounted_callback         (NautilusVolumeMonitor    *monitor,
-						   NautilusVolume           *volume,
+static void     volume_unmounted_callback         (GnomeVFSVolumeMonitor    *monitor,
+						   GnomeVFSVolume           *volume,
+						   NautilusApplication      *application);
+static void     volume_mounted_callback           (GnomeVFSVolumeMonitor    *monitor,
+						   GnomeVFSVolume           *volume,
 						   NautilusApplication      *application);
 static void     update_session                    (gpointer                  callback_data);
 static void     init_session                      (void);
@@ -152,6 +155,13 @@ nautilus_application_get_spatial_window_list (void)
 	return nautilus_application_spatial_window_list;
 }
 
+static CORBA_Object
+create_object_shortcut (const char *iid,
+			gpointer callback_data)
+{
+	return create_object (BONOBO_OBJREF (callback_data), iid, NULL);
+}
+
 static void
 nautilus_application_instance_init (NautilusApplication *application)
 {
@@ -163,8 +173,15 @@ nautilus_application_instance_init (NautilusApplication *application)
 	 * used anymore */
 
 	/* Watch for volume unmounts so we can close open windows */
-	g_signal_connect_object (nautilus_volume_monitor_get (), "volume_unmounted",
+	g_signal_connect_object (gnome_vfs_get_volume_monitor (), "volume_unmounted",
 				 G_CALLBACK (volume_unmounted_callback), application, 0);
+	g_signal_connect_object (gnome_vfs_get_volume_monitor (), "volume_mounted",
+				 G_CALLBACK (volume_mounted_callback), application, 0);
+
+	nautilus_bonobo_register_activation_shortcut (NAUTILUS_ICON_VIEW_IID, create_object_shortcut, application);
+	nautilus_bonobo_register_activation_shortcut (NAUTILUS_DESKTOP_ICON_VIEW_IID, create_object_shortcut, application);
+	nautilus_bonobo_register_activation_shortcut (NAUTILUS_LIST_VIEW_IID, create_object_shortcut, application);
+	nautilus_bonobo_register_activation_shortcut (SEARCH_LIST_VIEW_IID, create_object_shortcut, application);
 }
 
 NautilusApplication *
@@ -188,6 +205,11 @@ nautilus_application_destroy (BonoboObject *object)
 
 	application = NAUTILUS_APPLICATION (object);
 
+	nautilus_bonobo_unregister_activation_shortcut (NAUTILUS_ICON_VIEW_IID);
+	nautilus_bonobo_unregister_activation_shortcut (NAUTILUS_DESKTOP_ICON_VIEW_IID);
+	nautilus_bonobo_unregister_activation_shortcut (NAUTILUS_LIST_VIEW_IID);
+	nautilus_bonobo_unregister_activation_shortcut (SEARCH_LIST_VIEW_IID);
+	
 	nautilus_bookmarks_exiting ();
 	
 	bonobo_object_unref (application->undo_manager);
@@ -331,6 +353,7 @@ migrate_old_nautilus_files (void)
 				      _("Migrated old desktop"),
 				      NULL);
 	}
+	g_free (old_desktop_dir);
 	g_free (migrated_file);
 }
 
@@ -434,6 +457,7 @@ nautilus_application_startup (NautilusApplication *application,
 			      gboolean no_default_window,
 			      gboolean no_desktop,
 			      gboolean do_first_time_druid_check,
+			      gboolean browser_window,
 			      const char *geometry,
 			      const char *urls[])
 {
@@ -607,10 +631,10 @@ nautilus_application_startup (NautilusApplication *application,
 	  	/* Create the other windows. */
 		if (urls != NULL) {
 			url_list = nautilus_make_uri_list_from_shell_strv (urls);
-			Nautilus_Shell_open_windows (shell, url_list, corba_geometry, &ev);
+			Nautilus_Shell_open_windows (shell, url_list, corba_geometry, browser_window, &ev);
 			CORBA_free (url_list);
 		} else if (!no_default_window) {
-			Nautilus_Shell_open_default_window (shell, corba_geometry, &ev);
+			Nautilus_Shell_open_default_window (shell, corba_geometry, browser_window, &ev);
 		}
 		
 		/* Add ourselves to the session */
@@ -855,20 +879,20 @@ find_parent_spatial_window (NautilusSpatialWindow *window)
 }
 
 void
-nautilus_application_close_with_parent_windows (NautilusSpatialWindow *window)
+nautilus_application_close_parent_windows (NautilusSpatialWindow *window)
 {
 	NautilusSpatialWindow *parent_window;
+	NautilusSpatialWindow *new_parent_window;
 
 	g_return_if_fail (NAUTILUS_IS_SPATIAL_WINDOW (window));
 
 	parent_window = find_parent_spatial_window (window);
-	nautilus_window_close (NAUTILUS_WINDOW (window));
-	window = parent_window;
 	
 	while (parent_window) {
-		parent_window = find_parent_spatial_window (window);
-		nautilus_window_close (NAUTILUS_WINDOW (window));
-		window = parent_window;
+		
+		new_parent_window = find_parent_spatial_window (parent_window);
+		nautilus_window_close (NAUTILUS_WINDOW (parent_window));
+		parent_window = new_parent_window;
 	}
 }
 
@@ -1067,37 +1091,54 @@ window_can_be_closed (NautilusWindow *window)
 	return FALSE;
 }
 
+static void
+volume_mounted_callback (GnomeVFSVolumeMonitor *monitor,
+			 GnomeVFSVolume *volume,
+			 NautilusApplication *application)
+{
+	char *activation_uri;
+	NautilusDirectory *directory;
+		
+	activation_uri = gnome_vfs_volume_get_activation_uri (volume);
+	directory = nautilus_directory_get_existing (activation_uri);
+	g_free (activation_uri);
+	if (directory != NULL) {
+		nautilus_directory_force_reload (directory);
+		nautilus_directory_unref (directory);
+	}
+}
+
 /* Called whenever a volume is unmounted. Check and see if there are any windows open
  * displaying contents on the volume. If there are, close them.
  * It would also be cool to save open window and position info.
  */
 static void
-volume_unmounted_callback (NautilusVolumeMonitor *monitor, NautilusVolume *volume,
+volume_unmounted_callback (GnomeVFSVolumeMonitor *monitor,
+			   GnomeVFSVolume *volume,
 			   NautilusApplication *application)
 {
 	GList *window_list, *node, *close_list;
 	NautilusWindow *window;
-	char *uri;
-	char *path;
+	char *uri, *activation_uri;
 		
 	close_list = NULL;
 	
 	/* Check and see if any of the open windows are displaying contents from the unmounted volume */
 	window_list = nautilus_application_get_window_list ();
-	
+
+	activation_uri = gnome_vfs_volume_get_activation_uri (volume);
 	/* Construct a list of windows to be closed. Do not add the non-closable windows to the list. */
 	for (node = window_list; node != NULL; node = node->next) {
 		window = NAUTILUS_WINDOW (node->data);
 		if (window != NULL && window_can_be_closed (window)) {
 			uri = nautilus_window_get_location (window);
-			path = gnome_vfs_get_local_path_from_uri (uri);
-			if (eel_str_has_prefix (path, nautilus_volume_get_mount_path (volume))) {
+			if (eel_str_has_prefix (uri, activation_uri)) {
 				close_list = g_list_prepend (close_list, window);
 			}
-			g_free (path);
 			g_free (uri);
 		}
 	}
+	g_free (activation_uri);
 		
 	/* Handle the windows in the close list. */
 	for (node = close_list; node != NULL; node = node->next) {
