@@ -239,11 +239,11 @@ nautilus_spatial_window_show (GtkWidget *widget)
 }
 
 static void
-file_menu_close_with_parent_windows_callback (BonoboUIComponent *component, 
+file_menu_close_parent_windows_callback (BonoboUIComponent *component, 
 					      gpointer user_data, 
 					      const char *verb)
 {
-	nautilus_application_close_with_parent_windows (NAUTILUS_SPATIAL_WINDOW (user_data));
+	nautilus_application_close_parent_windows (NAUTILUS_SPATIAL_WINDOW (user_data));
 }
 
 static void
@@ -275,7 +275,7 @@ real_merge_menus (NautilusWindow *nautilus_window)
 {
 	NautilusSpatialWindow *window;
 	BonoboUIVerb verbs [] = {
-		BONOBO_UI_VERB ("Close With Parents", file_menu_close_with_parent_windows_callback),
+		BONOBO_UI_VERB ("Close Parent Folders", file_menu_close_parent_windows_callback),
 		BONOBO_UI_VERB_END
 	};
 	
@@ -312,6 +312,17 @@ real_delete_event (GtkWidget *window, GdkEventAny *event)
 	return FALSE;
 }
 
+static void 
+real_get_default_size(NautilusWindow *window, guint *default_width, guint *default_height)
+{
+   if(default_width) {
+      *default_width = NAUTILUS_SPATIAL_WINDOW_DEFAULT_WIDTH;
+   }
+   if(default_height) {
+      *default_height = NAUTILUS_SPATIAL_WINDOW_DEFAULT_HEIGHT;	
+   }
+}
+
 static void
 nautilus_spatial_window_instance_init (NautilusSpatialWindow *window)
 {
@@ -346,4 +357,5 @@ nautilus_spatial_window_class_init (NautilusSpatialWindowClass *class)
 		real_set_content_view_widget;
 	GTK_WIDGET_CLASS (class)->delete_event =
 		real_delete_event;
+	NAUTILUS_WINDOW_CLASS(class)->get_default_size = real_get_default_size;
 }
